"""Auto-generated model profiles.

DO NOT EDIT THIS FILE MANUALLY.
This file is generated by the langchain-profiles CLI tool.

It contains data derived from the models.dev project.

Source: https://github.com/sst/models.dev
License: MIT License

To update these data, refer to the instructions here:

https://docs.langchain.com/oss/python/langchain/models#updating-or-overwriting-profile-data
"""

from typing import Any

_PROFILES: dict[str, dict[str, Any]] = {
    "chatgpt-image-latest": {
        "name": "chatgpt-image-latest",
        "release_date": "2025-12-16",
        "last_updated": "2025-12-16",
        "open_weights": False,
        "max_input_tokens": 0,
        "max_output_tokens": 0,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": True,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-3.5-turbo": {
        "name": "GPT-3.5-turbo",
        "release_date": "2023-03-01",
        "last_updated": "2023-11-06",
        "open_weights": False,
        "max_input_tokens": 16385,
        "max_output_tokens": 4096,
        "text_inputs": True,
        "image_inputs": False,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "structured_output": False,
        "attachment": False,
        "temperature": True,
        "image_url_inputs": False,
        "pdf_inputs": False,
        "pdf_tool_message": False,
        "image_tool_message": False,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4": {
        "name": "GPT-4",
        "release_date": "2023-11-06",
        "last_updated": "2024-04-09",
        "open_weights": False,
        "max_input_tokens": 8192,
        "max_output_tokens": 8192,
        "text_inputs": True,
        "image_inputs": False,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": False,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4-turbo": {
        "name": "GPT-4 Turbo",
        "release_date": "2023-11-06",
        "last_updated": "2024-04-09",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 4096,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": False,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4.1": {
        "name": "GPT-4.1",
        "release_date": "2025-04-14",
        "last_updated": "2025-04-14",
        "open_weights": False,
        "max_input_tokens": 1047576,
        "max_output_tokens": 32768,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4.1-mini": {
        "name": "GPT-4.1 mini",
        "release_date": "2025-04-14",
        "last_updated": "2025-04-14",
        "open_weights": False,
        "max_input_tokens": 1047576,
        "max_output_tokens": 32768,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4.1-nano": {
        "name": "GPT-4.1 nano",
        "release_date": "2025-04-14",
        "last_updated": "2025-04-14",
        "open_weights": False,
        "max_input_tokens": 1047576,
        "max_output_tokens": 32768,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4o": {
        "name": "GPT-4o",
        "release_date": "2024-05-13",
        "last_updated": "2024-08-06",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4o-2024-05-13": {
        "name": "GPT-4o (2024-05-13)",
        "release_date": "2024-05-13",
        "last_updated": "2024-05-13",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 4096,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4o-2024-08-06": {
        "name": "GPT-4o (2024-08-06)",
        "release_date": "2024-08-06",
        "last_updated": "2024-08-06",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4o-2024-11-20": {
        "name": "GPT-4o (2024-11-20)",
        "release_date": "2024-11-20",
        "last_updated": "2024-11-20",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-4o-mini": {
        "name": "GPT-4o mini",
        "release_date": "2024-07-18",
        "last_updated": "2024-07-18",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5": {
        "name": "GPT-5",
        "release_date": "2025-08-07",
        "last_updated": "2025-08-07",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5-chat-latest": {
        "name": "GPT-5 Chat (latest)",
        "release_date": "2025-08-07",
        "last_updated": "2025-08-07",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": False,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5-codex": {
        "name": "GPT-5-Codex",
        "release_date": "2025-09-15",
        "last_updated": "2025-09-15",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": False,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5-mini": {
        "name": "GPT-5 Mini",
        "release_date": "2025-08-07",
        "last_updated": "2025-08-07",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5-nano": {
        "name": "GPT-5 Nano",
        "release_date": "2025-08-07",
        "last_updated": "2025-08-07",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5-pro": {
        "name": "GPT-5 Pro",
        "release_date": "2025-10-06",
        "last_updated": "2025-10-06",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 272000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.1": {
        "name": "GPT-5.1",
        "release_date": "2025-11-13",
        "last_updated": "2025-11-13",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.1-chat-latest": {
        "name": "GPT-5.1 Chat",
        "release_date": "2025-11-13",
        "last_updated": "2025-11-13",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.1-codex": {
        "name": "GPT-5.1 Codex",
        "release_date": "2025-11-13",
        "last_updated": "2025-11-13",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.1-codex-max": {
        "name": "GPT-5.1 Codex Max",
        "release_date": "2025-11-13",
        "last_updated": "2025-11-13",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.1-codex-mini": {
        "name": "GPT-5.1 Codex mini",
        "release_date": "2025-11-13",
        "last_updated": "2025-11-13",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.2": {
        "name": "GPT-5.2",
        "release_date": "2025-12-11",
        "last_updated": "2025-12-11",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.2-chat-latest": {
        "name": "GPT-5.2 Chat",
        "release_date": "2025-12-11",
        "last_updated": "2025-12-11",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.2-codex": {
        "name": "GPT-5.2 Codex",
        "release_date": "2025-12-11",
        "last_updated": "2025-12-11",
        "open_weights": False,
        "max_input_tokens": 400000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.2-pro": {
        "name": "GPT-5.2 Pro",
        "release_date": "2025-12-11",
        "last_updated": "2025-12-11",
        "open_weights": False,
        "max_input_tokens": 272000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.3-chat-latest": {
        "name": "GPT-5.3 Chat (latest)",
        "release_date": "2026-03-03",
        "last_updated": "2026-03-03",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 16384,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": True,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.3-codex": {
        "name": "GPT-5.3 Codex",
        "release_date": "2026-02-05",
        "last_updated": "2026-02-05",
        "open_weights": False,
        "max_input_tokens": 400000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.3-codex-spark": {
        "name": "GPT-5.3 Codex Spark",
        "release_date": "2026-02-05",
        "last_updated": "2026-02-05",
        "open_weights": False,
        "max_input_tokens": 128000,
        "max_output_tokens": 32000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.4": {
        "name": "GPT-5.4",
        "release_date": "2026-03-05",
        "last_updated": "2026-03-05",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.4-mini": {
        "name": "GPT-5.4 mini",
        "release_date": "2026-03-17",
        "last_updated": "2026-03-17",
        "open_weights": False,
        "max_input_tokens": 400000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.4-nano": {
        "name": "GPT-5.4 nano",
        "release_date": "2026-03-17",
        "last_updated": "2026-03-17",
        "open_weights": False,
        "max_input_tokens": 400000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.4-pro": {
        "name": "GPT-5.4 Pro",
        "release_date": "2026-03-05",
        "last_updated": "2026-03-05",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.5": {
        "name": "GPT-5.5",
        "release_date": "2026-04-23",
        "last_updated": "2026-04-23",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.5-pro": {
        "name": "GPT-5.5 Pro",
        "release_date": "2026-04-23",
        "last_updated": "2026-04-23",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.6": {
        "name": "GPT-5.6",
        "release_date": "2026-07-09",
        "last_updated": "2026-07-09",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.6-luna": {
        "name": "GPT-5.6 Luna",
        "release_date": "2026-07-09",
        "last_updated": "2026-07-09",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.6-sol": {
        "name": "GPT-5.6 Sol",
        "release_date": "2026-07-09",
        "last_updated": "2026-07-09",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-5.6-terra": {
        "name": "GPT-5.6 Terra",
        "release_date": "2026-07-09",
        "last_updated": "2026-07-09",
        "open_weights": False,
        "max_input_tokens": 1050000,
        "max_output_tokens": 128000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-image-1": {
        "name": "gpt-image-1",
        "release_date": "2025-04-24",
        "last_updated": "2025-04-24",
        "open_weights": False,
        "max_input_tokens": 0,
        "max_output_tokens": 0,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": False,
        "image_outputs": True,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-image-1-mini": {
        "name": "gpt-image-1-mini",
        "release_date": "2025-09-26",
        "last_updated": "2025-09-26",
        "open_weights": False,
        "max_input_tokens": 0,
        "max_output_tokens": 0,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": True,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-image-1.5": {
        "name": "gpt-image-1.5",
        "release_date": "2025-11-25",
        "last_updated": "2025-11-25",
        "open_weights": False,
        "max_input_tokens": 0,
        "max_output_tokens": 0,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": True,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "gpt-image-2": {
        "name": "gpt-image-2",
        "release_date": "2026-04-21",
        "last_updated": "2026-04-21",
        "open_weights": False,
        "max_input_tokens": 0,
        "max_output_tokens": 0,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": False,
        "image_outputs": True,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o1": {
        "name": "o1",
        "release_date": "2024-12-05",
        "last_updated": "2024-12-05",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o1-pro": {
        "name": "o1-pro",
        "release_date": "2025-03-19",
        "last_updated": "2025-03-19",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o3": {
        "name": "o3",
        "release_date": "2025-04-16",
        "last_updated": "2025-04-16",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "pdf_inputs": True,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o3-deep-research": {
        "name": "o3-deep-research",
        "release_date": "2024-06-26",
        "last_updated": "2024-06-26",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o3-mini": {
        "name": "o3-mini",
        "release_date": "2024-12-20",
        "last_updated": "2025-01-29",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": False,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": False,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o3-pro": {
        "name": "o3-pro",
        "release_date": "2025-06-10",
        "last_updated": "2025-06-10",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o4-mini": {
        "name": "o4-mini",
        "release_date": "2025-04-16",
        "last_updated": "2025-04-16",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "structured_output": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "o4-mini-deep-research": {
        "name": "o4-mini-deep-research",
        "release_date": "2024-06-26",
        "last_updated": "2024-06-26",
        "open_weights": False,
        "max_input_tokens": 200000,
        "max_output_tokens": 100000,
        "text_inputs": True,
        "image_inputs": True,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": True,
        "tool_calling": True,
        "attachment": True,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "text-embedding-3-large": {
        "name": "text-embedding-3-large",
        "release_date": "2024-01-25",
        "last_updated": "2024-01-25",
        "open_weights": False,
        "max_input_tokens": 8191,
        "max_output_tokens": 3072,
        "text_inputs": True,
        "image_inputs": False,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": False,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "text-embedding-3-small": {
        "name": "text-embedding-3-small",
        "release_date": "2024-01-25",
        "last_updated": "2024-01-25",
        "open_weights": False,
        "max_input_tokens": 8191,
        "max_output_tokens": 1536,
        "text_inputs": True,
        "image_inputs": False,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": False,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
    "text-embedding-ada-002": {
        "name": "text-embedding-ada-002",
        "release_date": "2022-12-15",
        "last_updated": "2022-12-15",
        "open_weights": False,
        "max_input_tokens": 8192,
        "max_output_tokens": 1536,
        "text_inputs": True,
        "image_inputs": False,
        "audio_inputs": False,
        "video_inputs": False,
        "text_outputs": True,
        "image_outputs": False,
        "audio_outputs": False,
        "video_outputs": False,
        "reasoning_output": False,
        "tool_calling": False,
        "attachment": False,
        "temperature": False,
        "image_url_inputs": True,
        "pdf_inputs": True,
        "pdf_tool_message": True,
        "image_tool_message": True,
        "tool_choice": True,
        "tool_call_streaming": True,
    },
}
